import pytest


def test_flow_rates(api, settings):
    """
    This will test supported Flow Rate
        - unit (Union[pps, bps, kbps, mbps, gbps, line]): The value is a unit of this
        - value (Union[float, int]): The actual rate
    """
    config = api.config()

    tx, rx = config.ports.port(name="tx", location=settings.ports[0]).port(
        name="rx", location=settings.ports[1]
    )

    l1 = config.layer1.layer1()[0]
    l1.name = "l1"
    l1.port_names = [rx.name, tx.name]
    l1.media = settings.media
    l1.speed = settings.speed

    config.options.port_options.location_preemption = True

    rate_line, rate_pps, rate_bps, rate_kbps, rate_gbps = (
        config.flows.flow(name="rate_line")
        .flow(name="rate_pps")
        .flow(name="rate_bps")
        .flow(name="rate_kbps")
        .flow(name="rate_gbps")
    )

    rate_line.tx_rx.port.tx_name = tx.name
    rate_line.tx_rx.port.rx_name = rx.name
    rate_line.rate.percentage = 100

    rate_pps.tx_rx.port.tx_name = tx.name
    rate_pps.tx_rx.port.rx_name = rx.name
    rate_pps.rate.pps = 2000

    rate_bps.tx_rx.port.tx_name = tx.name
    rate_bps.tx_rx.port.rx_name = rx.name
    rate_bps.rate.bps = 700

    rate_kbps.tx_rx.port.tx_name = tx.name
    rate_kbps.tx_rx.port.rx_name = rx.name
    rate_kbps.rate.kbps = 800

    rate_gbps.tx_rx.port.tx_name = tx.name
    rate_gbps.tx_rx.port.rx_name = rx.name
    rate_gbps.rate.gbps = 10

    api.set_config(config)


if __name__ == "__main__":
    pytest.main(["-s", __file__])



import time
import pytest


def test_flow_tracking_stats(api, utils):
    config = api.config()
    api._enable_flow_tracking(True)
    # ports
    p1, p2, p3, p4 = (
        config.ports.port(name="p1", location=utils.settings.ports[0])
        .port(name="p2", location=utils.settings.ports[1])
        .port(name="p3", location=utils.settings.ports[2])
        .port(name="p4", location=utils.settings.ports[3])
    )
    layer1 = config.layer1.layer1()[-1]
    layer1.port_names = [port.name for port in config.ports]
    layer1.speed = "speed_100_gbps"
    layer1.media = "copper"
    layer1.name = "test"

    config.options.port_options.location_preemption = True

    d1, d2, d3, d4 = (
        config.devices.device(name="Device1")
        .device(name="Device2")
        .device(name="Device3")
        .device(name="Device4")
    )
    eth1, eth2 = (
        d1.ethernets.add(),
        d2.ethernets.add(),
    )

    eth3, eth4 = (
        d3.ethernets.add(),
        d4.ethernets.add(),
    )
    eth1.connection.port_name, eth2.connection.port_name = p1.name, p2.name
    eth3.connection.port_name, eth4.connection.port_name = p3.name, p4.name

    # device1
    eth1.name = "Eth1"
    eth1.mac = "00:02:00:00:00:11"
    ip1 = eth1.ipv4_addresses.add()
    ip1.name = "ip1"
    ip1.address = "10.10.10.1"
    ip1.prefix = 32
    ip1.gateway = "10.10.10.2"

    # device2
    eth2.name = "Eth2"
    eth2.mac = "00:02:00:00:00:12"
    ip2 = eth2.ipv4_addresses.add()
    ip2.name = "ip2"
    ip2.address = "10.10.10.2"
    ip2.prefix = 32
    ip2.gateway = "10.10.10.1"

    # device3
    eth3.name = "Eth3"
    eth3.mac = "00:02:00:00:00:13"
    ip3 = eth3.ipv4_addresses.add()
    ip3.name = "ip3"
    ip3.address = "20.20.20.1"
    ip3.prefix = 32
    ip3.gateway = "20.20.20.2"

    # device4
    eth4.name = "Eth4"
    eth4.mac = "00:02:00:00:00:14"
    ipv4 = eth4.ipv4_addresses.add()
    ipv4.name = "ip4"
    ipv4.address = "20.20.20.2"
    ipv4.prefix = 32
    ipv4.gateway = "20.20.20.1"

    # traffic
    config.flows.flow(name="Full Mesh Traffic")
    flow = config.flows[-1]
    flow.metrics.enable = True
    flow.metrics.loss = True
    endpoints = [device.name for device in config.devices]
    flow.tx_rx.device.tx_names = endpoints
    flow.tx_rx.device.rx_names = endpoints
    flow.packet.ethernet().ipv4().udp()
    flow.packet[1]
    flow.size.fixed = 128
    flow.duration.fixed_packets.packets = 10000
    flow.rate.percentage = 1
    api.set_config(config)

    print("Starting all protocols ...")  
    ps = api.control_state()
    ps.choice = ps.PROTOCOL
    ps.protocol.choice = ps.protocol.ALL
    ps.protocol.all.state = ps.protocol.all.START
    res = api.set_control_state(ps)
    if len(res.warnings) > 0:
        print("Warnings: {}".format(res.warnings))
    time.sleep(5)

    print("Start Traffic ...")
    # start traffic
    control_state = api.control_state()
    control_state.choice = control_state.TRAFFIC
    control_state.traffic.choice = control_state.traffic.FLOW_TRANSMIT
    control_state.traffic.flow_transmit.state = control_state.traffic.flow_transmit.START  # noqa
    res = api.set_control_state(control_state)
    if len(res.warnings) > 0:
        print("Warnings: {}".format(res.warnings))

    # check stats
    time.sleep(5)
    config = api.get_config()
    request = api.metrics_request()
    request.choice = request.FLOW
    results = api.get_metrics(request).flow_metrics
    assert len(results) == 12

    print("Stop all protocols ...")
    ps = api.control_state()
    ps.choice = ps.PROTOCOL
    ps.protocol.choice = ps.protocol.ALL
    ps.protocol.all.state = ps.protocol.all.STOP
    res = api.set_control_state(ps)
    if len(res.warnings) > 0:
        print("Warnings: {}".format(res.warnings))
    time.sleep(5)

    print("Stop Traffic ...")
    # stop traffic
    control_state = api.control_state()
    control_state.choice = control_state.TRAFFIC
    control_state.traffic.choice = control_state.traffic.FLOW_TRANSMIT
    control_state.traffic.flow_transmit.state = control_state.traffic.flow_transmit.STOP  # noqa
    res = api.set_control_state(control_state)
    if len(res.warnings) > 0:
        print("Warnings: {}".format(res.warnings))



@pytest.mark.skip(reason="run for 8 ports")
def test_flow_tracking_stats_8_ports(api):
    config = api.config()
    api._enable_flow_tracking(True)
    locations = [
        "localuhd/25",
        "localuhd/26",
        "localuhd/27",
        "localuhd/28",
        "localuhd/29",
        "localuhd/30",
        "localuhd/31",
        "localuhd/32",
    ]
    # ports
    for i in range(8):
        config.ports.port(name="p{}".format(i + 1), location=locations[i])
    layer1 = config.layer1.layer1()[-1]
    layer1.port_names = [port.name for port in config.ports]
    layer1.speed = "speed_100_gbps"
    layer1.media = "fiber"
    layer1.name = "test"

    config.options.port_options.location_preemption = True

    ports = [port for port in config.ports]

    for i in range(1, 9, 2):
        d = config.devices.device(name="Device{}".format(i))[-1]
        eth = d.ethernets.add()
        eth.connection.port_name = ports[i - 1].name
        eth.name = "Eth{}".format(i)
        eth.mac = "00:02:00:00:00:1{}".format(i)
        ip = eth.ipv4_addresses.add()
        ip.name = "ip{}".format(i)
        ip.address = "10.10.10.{}".format(i)
        ip.prefix = 32
        ip.gateway = "10.10.10.{}".format(i + 1)

    for i in range(1, 9, 2):
        d = config.devices.device(name="Device{}".format(i + 1))[-1]
        eth = d.ethernets.add()
        eth.connection.port_name = ports[i].name
        eth.name = "Eth{}".format(i + 1)
        eth.mac = "00:02:00:00:00:1{}".format(i + 1)
        ip = eth.ipv4_addresses.add()
        ip.name = "ip{}".format(i + 1)
        ip.address = "10.10.10.{}".format(i + 1)
        ip.prefix = 32
        ip.gateway = "10.10.10.{}".format(i)

    # traffic
    config.flows.flow(name="Full Mesh Traffic")
    flow = config.flows[-1]
    flow.metrics.enable = True
    flow.metrics.loss = True
    endpoints = [device.name for device in config.devices]
    flow.tx_rx.device.tx_names = endpoints
    flow.tx_rx.device.rx_names = endpoints
    flow.packet.ethernet().ipv4().udp()
    flow.packet[1]
    flow.size.fixed = 128
    flow.duration.fixed_packets.packets = 100000
    flow.rate.percentage = 1
    api.set_config(config)

    print("Starting all protocols ...")
    ps = api.protocol_state()
    ps.state = ps.START
    api.set_protocol_state(ps)
    time.sleep(5)

    # start traffic
    ts = api.transmit_state()
    ts.state = ts.START
    api.set_transmit_state(ts)
    while True:
        if check_status(api):
            break
    # check stats
    time.sleep(5)
    config = api.get_config()
    request = api.metrics_request()
    request.choice = request.FLOW
    results = api.get_metrics(request).flow_metrics
    assert len(results) == 56


def check_status(api):
    time.sleep(5)
    fq = api.metrics_request()
    fq.choice = fq.FLOW
    metrics = api.get_metrics(fq).flow_metrics
    print(metrics)
    frames_sent = all([f.frames_tx > 0 for f in metrics])
    all_stopped = all([m.transmit == "stopped" for m in metrics])
    print(frames_sent, all_stopped)
    return frames_sent and all_stopped



import pytest


def test_update_flows(api, b2b_raw_config, utils):
    """
    This test is to validate update_flows API

    1. once initial config is set, the test will update rate on one flow
       and size on other flow and validates if the rate and size got updated
       on the fly.

    2. Negative tests include updating other properties which are not allowed
       currently and the server should throw and exception.
    """

    ports = b2b_raw_config.ports
    flow1 = b2b_raw_config.flows[0]
    flow1.name = "tx_flow1"
    flow1.packet.ethernet().ipv4()
    flow1.packet[0].src.value = "00:0c:29:1d:10:67"
    flow1.packet[0].dst.value = "00:0c:29:1d:10:71"
    flow1.packet[1].src.value = "10.10.10.1"
    flow1.packet[1].dst.value = "10.10.10.2"
    flow2 = b2b_raw_config.flows.flow()[-1]
    flow2.name = "tx_flow2"
    flow2.tx_rx.port.tx_name = ports[0].name
    flow2.tx_rx.port.rx_name = ports[1].name

    flow3 = b2b_raw_config.flows.flow()[-1]
    flow3.name = "tx_flow3"
    flow3.tx_rx.port.tx_name = ports[0].name
    flow3.tx_rx.port.rx_name = ports[1].name

    flow1.duration.fixed_packets.packets = 1000
    flow1.size.fixed = 1000
    flow1.duration.choice = flow1.duration.CONTINUOUS
    flow1.rate.pps = 1000

    flow2.duration.fixed_packets.packets = 1000
    flow2.size.fixed = 1000
    flow2.duration.choice = flow1.duration.CONTINUOUS
    flow2.rate.pps = 1000

    flow3.duration.fixed_packets.packets = 1000
    flow3.size.fixed = 1000
    flow3.duration.choice = flow3.duration.CONTINUOUS
    flow3.rate.pps = 1000

    flow1.metrics.enable = True
    flow1.metrics.loss = True

    flow3.metrics.enable = True
    flow3.metrics.loss = True

    api.set_config(b2b_raw_config)

    utils.start_traffic(api, b2b_raw_config, start_capture=False)

    utils.wait_for(
        lambda: stats_ok(api, 2 * 1000, utils), "stats to be as expected"
    )

    req = api.config_update().flows
    req.property_names = [req.RATE, req.SIZE]

    update_flow1 = b2b_raw_config.flows[0]
    update_flow1.rate.pps = 2000
    req.flows.append(update_flow1)

    update_flow2 = b2b_raw_config.flows[2]
    update_flow2.size.fixed = 2000
    req.flows.append(update_flow2)

    api.update_flows(req)

    # update size validation
    validate_config(api, "tx_flow3", 2000)
    import time

    time.sleep(5)

    # update rate validation total 3000 = 1000(tx_flow1) + 2000(tx_flow3)
    utils.wait_for(
        lambda: stats_ok(api, 3000, utils), "stats to be as expected"
    )

    # Negative test
    req = api.config_update().flows
    req.property_names = [req.RATE, req.SIZE]

    update_flow1 = b2b_raw_config.flows[0]
    update_flow1.tx_rx.port.tx_name = "rx"
    update_flow1.tx_rx.port.rx_name = "tx"
    update_flow1.packet.ethernet().ipv4()
    update_flow1.packet[0].src.value = "00:0c:29:1d:10:98"
    update_flow1.packet[0].dst.value = "00:0c:29:1e:10:71"
    update_flow1.packet[1].src.value = "20.20.20.1"
    update_flow1.packet[1].dst.value = "10.20.30.2"
    req.flows.append(update_flow1)

    try:
        api.update_flows(req)
    except Exception as e:
        assert (
            "tx_rx property update is not supported on flow tx_flow1"
            in ",".join(e.message)
        )
        assert (
            "packet property update is not supported on flow tx_flow1"
            in ",".join(e.message)
        )

    utils.stop_traffic(api, b2b_raw_config)


def validate_config(api, flow_name, updated_size):
    assert (
        api._ixnetwork.Traffic.TrafficItem.find(Name=flow_name)
        .HighLevelStream.find()
        .FrameSize.FixedSize
    ) == updated_size


def stats_ok(api, framerate, utils):
    """
    Returns true if stats are as expected, false otherwise.
    """
    _, flow_stats = utils.get_all_stats(api)

    frame_rate = sum([f.frames_tx_rate for f in flow_stats])

    return frame_rate == framerate


if __name__ == "__main__":
    pytest.main(["-s", __file__])



def test_flow_duration(api, utils):
    """
    This will test different transmit durations:
    [1] Fixed : A fixed number of packets will be transmitted after which the flow will stop
                If the number of packets is set to 0 the flow will not stop
        Args
        ----
        - packets (int): Stop transmit of the flow after this number of packets. A value of 0 means that the flow will not stop transmitting
        - gap (int): The minimum gap between packets expressed as bytes
        - delay (int): The delay before starting transmission of packets
        - delay_unit (Union[bytes, nanoseconds]): The delay expressed as a number of this value

    [2] Burst : A continuous burst of packets that will not automatically stop
        Args
        ----
        - packets (int): The number of packets transmitted per burst
        - gap (int): The minimum gap between packets expressed as bytes
        - inter_burst_gap (int): The gap between the transmission of each burst. A value of 0 means there is no gap between bursts
        - inter_burst_gap_unit (Union[bytes, nanoseconds]): The inter burst gap expressed as a number of this value
    """

    config = api.config()
    config.ports.port(name="tx").port(name="rx")
    tx_port, rx_port = config.ports
    tx_port.location = utils.settings.ports[0]
    rx_port.location = utils.settings.ports[1]
    l1, l2 = config.layer1.layer1(name="l10").layer1(name="l11")
    l1.port_names, l2.port_names = [tx_port.name], [rx_port.name]
    l1.media, l1.media = utils.settings.media, utils.settings.media
    l1.speed, l2.speed = utils.settings.speed, utils.settings.speed
    config.options.port_options.location_preemption = True
    c, fp, fs, b = config.flows.flow().flow().flow().flow()

    # Test for Continuous Flow
    c.name = "Continuous Duration"
    c.packet.ethernet().vlan().ipv4()
    c.duration.choice = c.duration.CONTINUOUS
    c.tx_rx.port.tx_name = tx_port.name
    c.tx_rx.port.rx_name = rx_port.name

    # Test for Fix packet with Gap and Delay
    fp.name = "Fixed Packet Duration"
    fp.tx_rx.port.tx_name = tx_port.name
    fp.tx_rx.port.rx_name = rx_port.name
    fp.packet.ethernet().vlan().ipv4()
    fp.duration.fixed_packets.packets = 125
    fp.duration.fixed_packets.gap = 2
    fp.duration.fixed_packets.delay.bytes = 8

    # Test for Fix second with Gap and Delay
    fs.name = "Fixed Seconds Duration"
    fs.tx_rx.port.tx_name = tx_port.name
    fs.tx_rx.port.rx_name = rx_port.name
    fs.packet.ethernet().vlan().ipv4()
    fs.duration.fixed_seconds.seconds = 312
    fs.duration.fixed_seconds.gap = 2
    fs.duration.fixed_seconds.delay.bytes = 8

    # Test for Burst Duration with Gap and inter burst gap
    b.name = "Fixed Burst Duration"
    b.tx_rx.port.tx_name = tx_port.name
    b.tx_rx.port.rx_name = rx_port.name
    b.packet.ethernet().vlan().ipv4()
    b.duration.burst.packets = 700
    b.duration.burst.gap = 8
    b.duration.burst.inter_burst_gap.nanoseconds = 4

    api.set_config(config)
